app load ["Int", "Time", "Timer"];
use "wordring.sml";

type words       = string list
type min_overlap = int
type max_overlap = int
type ring_length = int
type overlap     = int
type ring        = (string * overlap) list

exception Wordring of string;

(* wordring_in f = Az f szvegfjlbl beolvasott adatokbl
                   kpzett feladvnyler ngyes
   wordring_in : string -> min_overlap * max_overlap * ring_length * words
*)
fun wordring_in f =
    let
	(* triple [a,b,c] = a hromelem lista elemeibl kpzett (a,b,c) hrmas
           triple : 'a list -> 'a * 'a * 'a
        *)
	fun triple [a,b,c] = (a,b,c)
	  | triple _ =
	      raise Wordring("Szintaktikusan hibs feladvnyler a(z) " ^
                               f ^ " fjlban.")

	(* fmString l2n = olyan fggvny, amely az l2n segdfggvnnyel a
                    fzr paramterbl kiolvassa a formz karakterekkel
                    elvlasztott egszeket, s az l2n-nel ellltott
                    kifejezst adja eredmnyl
           fmString : (int list -> 'a) -> string -> 'a
        *)
	fun fmString l2n = l2n o (List.mapPartial Int.fromString) o
			   (String.tokens Char.isSpace)

	(* inputLine s = az s folyam kvetkez sora
           inputLine : instream -> string
        *)
	fun inputLine s = TextIO.inputLine s

	(* wordlist_in s = az s folyam sszes sorbl ll lista,
                           az eredeti sorrend megrzsvel
           wordlist_in : instream -> string list
        *)
	fun wordlist_in s =
	    if TextIO.endOfStream s then []
	    else
		let
		    val line = inputLine s
		    val word = implode(List.filter Char.isAlpha (explode line))
		in
		    word :: wordlist_in s
		end (* let *)

	val istr      = TextIO.openIn f
	val (min_overlap, max_overlap, ring_length) =
                                             (fmString triple) (inputLine istr)
	val wordlist = wordlist_in istr
    in
	(min_overlap, max_overlap, ring_length, wordlist)
    end (* let *)

(* tab i = i db szkzbl ll fzr
   tab : int -> string
*)
fun tab 0 = "" | tab i = " " ^ tab(i-1)

(* wordring2string n ms = az ms szgyr szavait jelenti meg, soronknt
                          n karakternyi eltolssal
   wordring2string : int -> ring -> string
*)
fun wordring2string n [] = tab n ^ "|..." ^ "\n"
  | wordring2string n ((t,i)::ts) =
	tab n ^ t ^ "\n" ^ wordring2string (n + size t - i) ts

(* wordring_out(m, mss) = A feladvny mss listban brzolt megoldst
     olvashat formban kirja az m szveges llomnyba.
   wordring_out : string * ring list * ring_length -> unit
*)
fun wordring_out(m, mss) =
      let val ostr = TextIO.openOut m
	  val _ = TextIO.output(ostr, String.concat(map (wordring2string 0) mss))
      in
	 TextIO.closeOut ostr
      end (* let *)

(* megold(f, m) = Beolvas egy feladvnyt az f szveges llomnybl s
      kirja az sszes megoldst az m szveges llomnyba.  Ehhez
      termszetesen szksge van a wordring fggvnyre.  Eredmnye az f
      nevet s a megoldsok szmt tartalmaz fzr.
   megold : string * string -> string
*)
fun megold (f, m) =
    let val t = wordring_in f
	val mss = wordring t
	val _ = wordring_out(m, mss)
    in
	"Feladvny: " ^ f ^ ", " ^ Int.toString(length mss) ^
          " megolds, " ^ "\n"
    end (* let *)

(* stopper(f, m) = Beolvas egy feladvnyt az f szveges llomnybl s
      kirja az sszes megoldst az m szveges llomnyba.  Ehhez
      termszetesen szksge van a wordring fggvnyre. Eredmnye az f
      nevet, a megoldsok szmt s a futsi idt mp-ben tartalmaz
      fzr.
   stopper : string * string -> string
*)
fun stopper (f, m) =
    let val t = wordring_in f
	val start = Timer.startCPUTimer()
	val mss = wordring t
	val {usr=tim,...} = Timer.checkCPUTimer start
	val _ = wordring_out(m, mss)
    in
	"Feladvny: " ^ f ^ ", " ^ Int.toString(length mss) ^
	  " megolds, " ^ "id: " ^ Time.fmt 2 tim ^ " mp\n"
    end (* let *)
